/*
 * Constructive Interference
 * an Arduino Game by Jon Leek (@jon_leek)
 * and Carol Mertz (@carolmertz)
 */
 
 
int targetR = 13;       //the pin that controls R for the target LED
int targetG = 12;       //the pin that controls G for the target LED
int targetB = 11;       //the pin that controls B for the target LED
int contPins[] = {A13,A14,A15};
int playerRGB[] = {4,3,2};

//Set start value of target light (NOTE: these values are overridden by nextPhase function)
int tR = 255; //random(0,255); //start value of target LEDs red
int tG = 255; //random(0,255); //start value of target LEDs green
int tB = 255; //random(0,255); //start value of target LEDs blue

int buzzer1 = 37; // The pin that controls P1 Buzzer
int buzzer2 = 36; // The pin that controls P2 Buzzer
int buzzer3 = 35; // The pin that controls P3 Buzzer

int playerPhase = 0;

// Define the LED digit patters, from 0 - 9
// Note that these patterns are for common anode displays
// For common cathode displays, change the 1's to 0's and 0's to 1's
// 1 = LED on, 0 = LED off, in this order:
// Arduino pin: 23,24,25,26,27,28,29
byte seven_seg_digits[10][7] = { 
{ 0,0,0,0,0,0,1 },  // = 0
{ 1,0,0,1,1,1,1 },  // = 1
{ 0,0,1,0,0,1,0 },  // = 2
{ 0,0,0,0,1,1,0 },  // = 3
{ 1,0,0,1,1,0,0 },  // = 4
{ 0,1,0,0,1,0,0 },  // = 5
{ 0,1,0,0,0,0,0 },  // = 6
{ 0,0,0,1,1,1,1 },  // = 7
{ 0,0,0,0,0,0,0 },  // = 8
{ 0,0,0,1,1,0,0 }   // = 9
};

void setup() {

  //Init serial monitor for debug
  //Serial.begin(9600);

  //Initialize all utilized data pins
  
  pinMode(targetR, OUTPUT);
  pinMode(targetB, OUTPUT);
  pinMode(targetG, OUTPUT);    
  
  pinMode(buzzer1, OUTPUT);
  pinMode(buzzer2, OUTPUT);
  pinMode(buzzer3, OUTPUT);

  //Set LED Number Pins
  pinMode(23, OUTPUT);
  pinMode(24, OUTPUT);
  pinMode(25, OUTPUT);
  pinMode(26, OUTPUT);
  pinMode(27, OUTPUT);
  pinMode(28, OUTPUT);
  pinMode(29, OUTPUT);
  pinMode(30, OUTPUT);   
  writeDot(1);  // start with the "dot" off

  //Set random seed to analog noise from unused "0" input on Mega board
  randomSeed(analogRead(0));

  //Set initial RGB variables for target light color (NOTE: these values are overridden by nextPhase function)
  setTarget(tR,tG,tB);

  //Call "nextPhase" function to advance the counter and randomize the target LED
  nextPhase();

}

void loop() {
  // Reads controllers
  int pot1 = analogRead(contPins[0]);
  int pot2 = analogRead(contPins[1]);
  int pot3 = analogRead(contPins[2]);

  // Maps controller values to 0-255 (RGB color scale)
  int red = map(pot1, 0, 1023, 0, 255);
  int blue = map(pot2, 0, 1023, 0, 255);
  int green = map(pot3, 0, 1023, 0, 255);
  setPlayer(red,blue,green);
  
  //calculate difference between controller values and target values, convert to positive value using "abs()"
  int intervalr = abs(red - tR);
  int intervalb = abs(blue - tB);
  int intervalg = abs(green - tG);

  // Set the maximum interval difference to begin buzzing sound
  int tonalMax = 100;

  //Play frequency ranging from 1000hz to 5000hz depending on each player's proximity to the target
  int mightyMightyBuzzToneR = map(intervalr, 0, tonalMax, 5000, 500);
  int mightyMightyBuzzToneG = map(intervalg, 0, tonalMax, 5000, 500);
  int mightyMightyBuzzToneB = map(intervalb, 0, tonalMax, 5000, 500);

  //probably should have made this a switch or a function or something but w/e we're noobs
    if (intervalb < tonalMax) {
    tone(buzzer1, mightyMightyBuzzToneB);
  } else {
    noTone(buzzer1);
  }
    if (intervalr < tonalMax) {
    tone(buzzer2, mightyMightyBuzzToneR);
  } else {
    noTone(buzzer2);
  }

    if (intervalg < tonalMax) {
    tone(buzzer3, mightyMightyBuzzToneG);
  } else {
    noTone(buzzer3);
  }


  
  // randomly sets target LED if player values are within a specified range of their target values
  int range = 50; 
  if ((intervalr) < range && (intervalb) < range && (intervalg) < range)
  {
    nextPhase();
  }

  //LED Number Display shows phase count and resets once players pass level 9
  if (playerPhase < 10) {
 sevenSegWrite(playerPhase); 
  }
  else {
    playerPhase = 1; 
  }

// //Debugging
//  Serial.print(intervalr);
//  Serial.print('\n');
//  Serial.print(intervalg);
//  Serial.print('\n');
//  Serial.print(intervalb);
//  Serial.print('\n');

}

  // Function to set target LED RGB values
void setTarget(int red, int green, int blue){
  analogWrite(targetR, red);
  analogWrite(targetG, green);
  analogWrite(targetB, blue);
  
}
// function to set player LED RGB values based on potentiometer input
 void setPlayer(int red, int green, int blue){
  analogWrite(playerRGB[0], constrain(red, 0, 255));
  analogWrite(playerRGB[1], constrain(blue, 0, 255));
  analogWrite(playerRGB[2], constrain(green, 0, 255));
}

//triggers when players hit the target. turns off target light, plays a 3-tone sound on all buzzers, then randomizes target light, and increases the phase count
void nextPhase() {
  
  setTarget(0,0,0);

  tone(buzzer1, 1000, 1000);
  tone(buzzer2, 2000, 1000);
  tone(buzzer3, 3000, 1000);
  delay(500);
  tone(buzzer1, 3000, 1000);
  tone(buzzer2, 1000, 1000);
  tone(buzzer3, 2000, 1000);
  delay(500);
  tone(buzzer1, 2000, 1000);
  tone(buzzer2, 3000, 1000);
  tone(buzzer3, 1000, 1000);
  delay(500);
  
  tR = random(0,255); //random value of target LEDs red
  tG = random(0,255); //random value of target LEDs blue
  tB = random(0,255); //random value of target LEDs green
  setTarget(tR,tG,tB);
  playerPhase ++;


}


//LED SETTINGS
void writeDot(byte dot) {
  digitalWrite(30, dot);
}
    
void sevenSegWrite(byte digit) {
  byte pin = 23;
  for (byte segCount = 0; segCount < 7; ++segCount) {
    digitalWrite(pin, seven_seg_digits[digit][segCount]);
    ++pin;
  }
}



